/*
 * Copyright 2020 ETSI
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.etsi.saref.server.resources;

import java.io.File;
import java.io.IOException;
import java.util.Collection;

import org.apache.jena.graph.Node;
import org.apache.jena.query.Dataset;
import org.apache.jena.query.text.EntityDefinition;
import org.apache.jena.query.text.TextDatasetFactory;
import org.apache.jena.query.text.TextIndex;
import org.apache.jena.query.text.TextIndexConfig;
import org.apache.jena.tdb.TDBFactory;
import org.apache.jena.vocabulary.RDFS;
import org.apache.lucene.analysis.en.EnglishAnalyzer;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.SimpleFSDirectory;

public class DatasetManager {

	private static final String NAME_DATASET = "dataset";
	private static final String NAME_TDB = "tdb";
	private static final String NAME_LUCENE = "lucene";
	static Boolean isDSCreated= false;
	private final File datasetDir;
	private final Dataset dataset;

	public DatasetManager(File targetDir) throws IOException {
		datasetDir = new File(targetDir, NAME_DATASET);
		File tdbFile = new File(datasetDir, NAME_TDB);
		File luceneFile = new File(datasetDir, NAME_LUCENE);
		Dataset base = TDBFactory.createDataset(tdbFile.getAbsolutePath());
		EntityDefinition entDef = new EntityDefinition("uri", "text", "graph");
		entDef.setPrimaryPredicate(RDFS.label.asNode());
		entDef.setUidField("uid");
		entDef.setLangField("lang");
		Collection<Node> textPredicates = entDef.getPredicates("text");
		textPredicates.add(RDFS.comment.asNode());
		textPredicates.add(RDFS.comment.asNode());
		TextIndexConfig config = new TextIndexConfig(entDef);
		config.setAnalyzer(new EnglishAnalyzer());
		config.setValueStored(true);
		Directory dir = new SimpleFSDirectory(luceneFile.toPath());
		TextIndex textIndex = TextDatasetFactory.createLuceneIndex(dir, config);
		dataset = TextDatasetFactory.create(base, textIndex);
	}

	public Dataset getDataset() {
		return dataset;
	}

}
